@@ -9,9 +9,11 @@ __all__ = []
 # should be sent.
 #
 import warnings
+from kafkacrypto.utils import kafkacrypto_hash_version
+warnings.warn("KafkaCrypto Version Hash: {}".format(kafkacrypto_hash_version()),category=RuntimeWarning)
 try:
   import confluent_kafka
-  from kafkacrypto.confluent_kafka_wrapper import KafkaConsumer,KafkaProducer,TopicPartition,TopicPartitionOffset,OffsetAndMetadata
+  from kafkacrypto.confluent_kafka_wrapper import KafkaConsumer,KafkaProducer,TopicPartition,TopicPartitionOffset,OffsetAndMetadata,OFFSET_BEGINNING,OFFSET_END
   warnings.warn("Using confluent_kafka: {}, librdkafka: {}".format(str(confluent_kafka.version()), str(confluent_kafka.libversion())),category=RuntimeWarning)
   # enable custom flush workaround for affected versions of librdkafka: 1.8.x
   # See https://github.com/edenhill/librdkafka/issues/3633
@@ -21,10 +23,16 @@ try:
 except ImportError:
   # fallback to kafka-python
   warnings.warn("No confluent_kafka package found. Falling back to kafka-python. It is highly, recommended that you install confluent_kafka and librdkafka for better performance, especially with large messages.",category=RuntimeWarning)
-  from kafkacrypto.kafka_python_wrapper import KafkaConsumer,KafkaProducer,TopicPartition,TopicPartitionOffset,OffsetAndMetadata
+  from kafkacrypto.kafka_python_wrapper import KafkaConsumer,KafkaProducer,TopicPartition,TopicPartitionOffset,OffsetAndMetadata,OFFSET_BEGINNING,OFFSET_END
+try:
+  # See keys.py
+  import oqs
+  warnings.warn("liboqs-python found! Enabling support for post-quantum cryptography.",category=RuntimeWarning)
+except ImportError:
+  pass
 del warnings
 
-__all__.extend(['KafkaConsumer', 'KafkaProducer', 'TopicPartition', 'TopicPartitionOffset', 'OffsetAndMetadata'])
+__all__.extend(['KafkaConsumer', 'KafkaProducer', 'TopicPartition', 'TopicPartitionOffset', 'OffsetAndMetadata', 'OFFSET_BEGINNING', 'OFFSET_END'])
 
 from kafkacrypto.message import KafkaCryptoMessage
 from kafkacrypto.crypto import KafkaCrypto
