@@ -169,6 +169,11 @@ class KafkaConsumer(Consumer):
     for oldk,newk in self.CONFIG_MAP.items():
       if newk in self.cf_config.keys():
         self.config[oldk] = self.cf_config[newk]
+    # Remove empty parameters
+    for k in list(self.cf_config.keys()):
+      if (self.cf_config[k] is None) or (isinstance(self.cf_config[k], (str,)) and len(self.cf_config[k]) == 0):
+        self._log.warning("Warning: Removing empty parameter %s passed to confluent wrapper", k)
+        self.cf_config.pop(k)
     super().__init__(self.cf_config)
     if topics:
       self.subscribe(topics)
@@ -397,6 +402,11 @@ class KafkaProducer(Producer):
     for oldk,newk in self.CONFIG_MAP.items():
       if newk in self.cf_config.keys():
         self.config[oldk] = self.cf_config[newk]
+    # Remove empty parameters
+    for k in list(self.cf_config.keys()):
+      if (self.cf_config[k] is None) or (isinstance(self.cf_config[k], (str,)) and len(self.cf_config[k]) == 0):
+        self._log.warning("Warning: Removing empty parameter %s passed to confluent wrapper", k)
+        self.cf_config.pop(k)
     self.messages_processed = 0
     self.messages_processed_lock = Lock()
     super().__init__(self.cf_config)
