@@ -13,7 +13,7 @@ class KafkaCryptoController(KafkaCryptoBase):
   """ A simple controller implementation, resigning requests for keys
       for a particular resource, using our key (signed by ROT), if
       request is signed by a known provisioner. To function properly,
-      and not miss messages, the provided KafkaConsumer must be 
+      and not miss messages, the provided KafkaConsumer must be
       configured so that client_id and group_id are both set to nodeID,
       and that enable_auto_commit (automatic committing) is False.
 
@@ -23,7 +23,7 @@ class KafkaCryptoController(KafkaCryptoBase):
                             handling crypto-keying messages. Should
                             not be used elsewhere, as this class
                             changes some configuration values.
-        kc (KafkaConsumer): Pre-initialized KafkaConsumer, ready for      
+        kc (KafkaConsumer): Pre-initialized KafkaConsumer, ready for
        	       	       	    handling crypto-keying messages. Should
                             not be used elsewhere, as this class
                             changes some configuration values.
@@ -40,7 +40,7 @@ class KafkaCryptoController(KafkaCryptoBase):
                             set_cryptokey)
            cryptokey (obj): Optional object implementing the
                             necessary public/private key functions
-                            (get/sign_spk,get/use_epk,
+                            (get/sign_spk,get/use_epks,
                             wrap/unwrap_opaque).
                             Set to None to load from the default
                             location in the configuration file.
@@ -65,10 +65,12 @@ class KafkaCryptoController(KafkaCryptoBase):
           k = pysodium.crypto_hash_sha256(provs[p])
           self._cryptostore.store_value(k,provs[p],section='allowlist')
           self._cryptostore.store_value(p,None,section='provisioners')
+      # allowlist entries are msgpack encoded single certificates (with public key in index 2)
       allowlist = self._cryptostore.load_section('allowlist',defaults=False)
       if not (allowlist is None):
         allowlist = allowlist.values()
       denylist = self._cryptostore.load_section('denylist',defaults=False)
+      # denylist entries are msgpack encoded single certificates (with public key in index 2)
       if not (denylist is None):
         denylist = denylist.values()
       provisioners = Provisioners(allowlist=allowlist, denylist=denylist)
@@ -122,7 +124,7 @@ class KafkaCryptoController(KafkaCryptoBase):
           elif topic == self.MGMT_TOPIC_CHAINS:
             # New candidate public key chain
             self._logger.info("Received new chain message: %s", msg)
-            if msg.key == self._cryptokey.get_spk():
+            if msg.key == bytes(self._cryptokey.get_spk()):
               self._logger.debug("Key matches ours. Validating Chain.")
               newchain = self._cryptoexchange.replace_spk_chain(msg.value)
               if not (newchain is None):
